package com.ether;

import com.medias.database.objects.Parametre;
import com.medias.database.objects.Plateforme;
import gov.noaa.pmel.sgt.Axis;
import gov.noaa.pmel.sgt.CartesianGraph;
import gov.noaa.pmel.sgt.ColorMap;
import gov.noaa.pmel.sgt.ContourLevels;
import gov.noaa.pmel.sgt.Graph;
import gov.noaa.pmel.sgt.GridAttribute;
import gov.noaa.pmel.sgt.IndexedColorMap;
import gov.noaa.pmel.sgt.JPane;
import gov.noaa.pmel.sgt.Layer;
import gov.noaa.pmel.sgt.LayerNotFoundException;
import gov.noaa.pmel.sgt.LineAttribute;
import gov.noaa.pmel.sgt.LineCartesianRenderer;
import gov.noaa.pmel.sgt.LineKey;
import gov.noaa.pmel.sgt.LinearTransform;
import gov.noaa.pmel.sgt.PlainAxis;
import gov.noaa.pmel.sgt.SGLabel;
import gov.noaa.pmel.sgt.StackedLayout;
import gov.noaa.pmel.sgt.TimeAxis;
import gov.noaa.pmel.sgt.dm.SGTMetaData;
import gov.noaa.pmel.sgt.dm.SimpleGrid;
import gov.noaa.pmel.sgt.dm.SimpleLine;
import gov.noaa.pmel.sgt.swing.JPlotLayout;
import gov.noaa.pmel.util.Dimension2D;
import gov.noaa.pmel.util.GeoDateArray;
import gov.noaa.pmel.util.Point2D;
import gov.noaa.pmel.util.Range2D;
import gov.noaa.pmel.util.SoTPoint;
import gov.noaa.pmel.util.SoTRange;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Required;

import java.awt.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Random;
import java.util.ResourceBundle;

/**
 * @author vmipsl
 * @date 05 sept. 2011
 */
public class EtherPlotContentServiceImpl
{
    /**
     * First layer contains axes, labels and the first set of data
     * Others layers contains only the other sets of data
     *
     * @param megapoliPlot
     * @param locale
     * @return
     */
    @Nullable
    public JPane createTimeSeriePlot( @NotNull final MegapoliPlot megapoliPlot, @NotNull final Integer plotWidth, @NotNull final Integer plotHeight, final Locale locale )
            throws ServiceException
    {
        final JPane mainPane = new JPane( "Time Serie Pane", new Dimension( plotWidth, plotHeight ) );
        mainPane.setLayout( new StackedLayout() );

        // TODO : revoir les tailles ! mettre en automatique par rapport à plotWidth et plotHeight
        /*
        * xsize, ysize are the width and height in physical units
        * of the Layer graphics region.
        *
        * xstart, xend are the start and end points for the TimeAxis
        * ystart, yend are the start and end points for the Y axis
        */
        final double xsize = 5.0;
        final double ysize = 3.5;
        final double xstart = 0.3;
        final double xend = xsize - xstart;
        final double ystart = 0.5;
        final double yend = ysize - ystart;
        // Init the index to pick up the colors in the array
        _indexColorArray = 0;

        final Pair<List, List> listPairOfPIdPIdForTwoAxes = extractListPairForAxes( megapoliPlot.getPfIdPIdList() );
        final List<Pair> pIdPIdListForFirstAxis = listPairOfPIdPIdForTwoAxes.getFirstValue();
        final List<Pair> pIdPIdListForSecondAxis = listPairOfPIdPIdForTwoAxes.getSecondValue();

        final List<SimpleLine> allLines = createLines( pIdPIdListForFirstAxis, megapoliPlot.getBeginDate(), megapoliPlot.getEndDate(), -2000, locale );
        final List<SimpleLine> allLinesForSecondAxis = createLines( pIdPIdListForSecondAxis, megapoliPlot.getBeginDate(), megapoliPlot.getEndDate(), 50, locale );
        List<SimpleLine> lines = extractRealLines( allLines );
        List<SimpleLine> linesForSecondAxis = extractRealLines( allLinesForSecondAxis );

        final List<SimpleLine> emptyLineToDisplayOnlyInLegend = createEmptyLinesList( allLines, allLinesForSecondAxis, lines );
        if( lines.isEmpty() )
        {
            if( linesForSecondAxis.isEmpty() )
                return null;
            else
            {
                lines = linesForSecondAxis;
                linesForSecondAxis = new ArrayList<SimpleLine>();
            }
        }

        final SimpleLine firstLine = lines.get( 0 );

        /** ********** AXIS RANGES ********* **/
        SoTRange xnRange = firstLine.getXRange();
        final SoTRange yRange = firstLine.getYRange();
        for( final SimpleLine line : lines )
        {
            yRange.add( line.getYRange() );
            xnRange.add( line.getXRange() );
        }

        for( final SimpleLine line : linesForSecondAxis )
            xnRange.add( line.getXRange() );

        if( xnRange.getStart().equals( xnRange.getEnd() ) )
            xnRange = Graph.computeTimeRange( xnRange, _intervalsNumber );

        /** ********************************** **/
        /** ************ FIRST AXIS ********** **/
        /** ********************************** **/
        final SoTRange ynRange = Graph.computeRange( yRange, _intervalsNumber );
        final SoTPoint originPoint = new SoTPoint( ynRange.getStart(), xnRange.getStart() );

        // Time axis is the same for the two y-axis
        final TimeAxis xTimeAxis = createXTimeAxis( xnRange, originPoint );
        final PlainAxis yLeftAxis = createYAxis( firstLine, ynRange, originPoint, null );
        final LinearTransform xt = new LinearTransform( new Range2D( xstart, xend ), xnRange );

        // GRAPH
        final CartesianGraph graph = createGraphForFirstLine( megapoliPlot, mainPane, xsize, ysize, ystart, yend, xTimeAxis, xt, firstLine, ynRange, yLeftAxis );
        graph.addXAxis( xTimeAxis );

        // LEGEND
        final LineKey lkey = createLineKey( megapoliPlot, ysize, graph.getLayer(), firstLine, graph );

        // OTHER LAYERS
        for( final SimpleLine otherLine : lines.subList( 1, lines.size() ) )
            addLineInNewGraph( megapoliPlot.getAxeType(), mainPane, xsize, ysize, otherLine, xt, (LinearTransform) graph.getYTransform(), lkey );


        /** ********************************** **/
        /** ************ SECOND AXIS ********* **/
        /** ********************************** **/
        if( !linesForSecondAxis.isEmpty() )
        {
            final SimpleLine firstLineForSecondAxis = linesForSecondAxis.get( 0 );
            /** ********** AXIS RANGES ********* **/
            final SoTRange yRangeForSecondAxis = firstLineForSecondAxis.getYRange();
            for( final SimpleLine line : linesForSecondAxis )
                yRangeForSecondAxis.add( line.getYRange() );

            final SoTRange ynRangeForSecondAxis = Graph.computeRange( yRangeForSecondAxis, _intervalsNumber );
            final SoTPoint endPointForSecondAxis = new SoTPoint( ynRangeForSecondAxis.getStart(), xnRange.getEnd() );
            final PlainAxis yRightAxis = createYAxis( firstLineForSecondAxis, ynRangeForSecondAxis, endPointForSecondAxis, Axis.POSITIVE_SIDE );

            // GRAPH
            final CartesianGraph graph2 = createGraphForFirstLine( megapoliPlot, mainPane, xsize, ysize, ystart, yend, xTimeAxis, xt, firstLineForSecondAxis, ynRangeForSecondAxis, yRightAxis );

            // LEGEND
            lkey.addLineGraph( (LineCartesianRenderer) graph2.getRenderer(), firstLineForSecondAxis.getKeyTitle() );

            // OTHER LAYERS
            for( final SimpleLine otherLine : linesForSecondAxis.subList( 1, linesForSecondAxis.size() ) )
                addLineInNewGraph( megapoliPlot.getAxeType(), mainPane, xsize, ysize, otherLine, xt, (LinearTransform) graph2.getYTransform(), lkey );
        }

        // Legend for empty lines
        displayLegendForEmptyLines( emptyLineToDisplayOnlyInLegend, lkey, mainPane, xsize, ysize, xt, (LinearTransform) graph.getYTransform(), megapoliPlot.getAxeType() );

        // Copyright
        displayCopyright( mainPane, xsize, ysize, xsize - 0.15, 0.1 );

        return mainPane;
    }

    @NotNull
    public JPane create2DPlot( @NotNull final MegapoliPlot megapoliPlot, @NotNull final Integer plotWidth, @NotNull final Integer plotHeight, @Nullable final Locale locale )
            throws ServiceException
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );

        final List<Double> parameterValues = (List<Double>) megapoliPlot.getData().getFirstArray();
        final List<Double> latitudeValues = (List<Double>) megapoliPlot.getData().getSecondArray();
        final List<Double> longitudeValues = (List<Double>) megapoliPlot.getData().getThirdArray();
        final double[] parameterValuesArray = EtherHelper.convertListDoubleToDoubleArray( parameterValues );
        final double[] latitudeValuesArray = EtherHelper.convertListDoubleToDoubleArray( latitudeValues );
        final double[] longitudeValuesArray = EtherHelper.convertListDoubleToDoubleArray( longitudeValues );

        // Create the grid
        final SGTMetaData xMeta = new SGTMetaData( bundle.getString( "plot.longitude" ), bundle.getString( "plot.degres" ) );
        final SGTMetaData yMeta = new SGTMetaData( bundle.getString( "plot.latitude" ), bundle.getString( "plot.degres" ) );

        final SimpleGrid simpleGrid = new SimpleGrid( parameterValuesArray, longitudeValuesArray, latitudeValuesArray, "Grid" );
        simpleGrid.setXMetaData( xMeta );
        simpleGrid.setYMetaData( yMeta );
        simpleGrid.setRealFullGrid( false );

        // Create the contour levels and color map
        final Double min = Collections.min( parameterValues );
        final Double max = Collections.max( parameterValues );
        final Double delta = ( max - min ) / _numberOfColorsFor2D;
        final Range2D datar = new Range2D( min, max, delta );
        final ContourLevels clevels = ContourLevels.getDefault( datar );
        final ColorMap cmap = createColorMap( datar );

        // Create a gridAttribute for contour data
        final GridAttribute gridAttribute = new GridAttribute( clevels );
        gridAttribute.setColorMap( cmap );
        gridAttribute.setStyle( GridAttribute.RASTER );

        // Create the legend
        final Pair<Integer, Integer> pIdPId = megapoliPlot.getPfIdPIdList().get( 0 );
        final Integer parameterId = pIdPId.getSecondValue();

        final Parametre parametre = _etherService.getParameterById( parameterId );
        if( null == parametre )
            throw new ServiceException( ServiceException.ServiceCode.PARAMETER_IS_NULL, new Throwable( ServiceException.ServiceCode.PARAMETER_IS_NULL.toString() ) );
        final String legend = parametre.getParametreNom() + " (" + parametre.getUnite().getUniteCode() + ")";

        // Create the plot
        final JPlotLayout jPlotLayout = new JPlotLayout( true, false, false, "JPlotLayout Pane", null, true );
        jPlotLayout.setEditClasses( false );
        jPlotLayout.addData( simpleGrid, gridAttribute, legend );
        jPlotLayout.setTitles( "", "", "" );
        jPlotLayout.setSize( new Dimension( plotWidth, plotHeight ) );

        try
        {
//            jPlotLayout.setKeyLayerSizeP( new Dimension2D( 12.0, 24.0 ) );
//            jPlotLayout.setKeyBoundsP( new Rectangle2D.Double( 0.01, 1.01, 5.98, 1.0 ) );
            final Layer keyLayer = jPlotLayout.getKeyPane().getLayer( "Key Layer" );
            jPlotLayout.add( keyLayer );
        }
        catch( LayerNotFoundException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.LEGEND_LAYER_NOT_FOUND, new Throwable( ServiceException.ServiceCode.LEGEND_LAYER_NOT_FOUND.toString() ) );
        }

        // Copyright
        displayCopyright( jPlotLayout, 1.0, 1.0, 1.15, 0.03 );

        // Period
        displayPeriod( jPlotLayout, megapoliPlot.getBeginDate(), megapoliPlot.getEndDate() );

        return jPlotLayout;
    }

    /**
     * This method creates a graph in a new layer with the neede axis and the first line to content
     *
     * @param megapoliPlot
     * @param mainPane     : the general pane
     * @param xsize
     * @param ysize
     * @param ystart
     * @param yend
     * @param xTimeAxis
     * @param xt
     * @param firstLine
     * @param ynRange
     * @param yAxis
     * @return
     */
    @NotNull
    private CartesianGraph createGraphForFirstLine( @NotNull final MegapoliPlot megapoliPlot, @NotNull final JPane mainPane, final double xsize, final double ysize, final double ystart, final double yend, @NotNull final TimeAxis xTimeAxis, @NotNull final LinearTransform xt, @NotNull final SimpleLine firstLine, @NotNull final SoTRange ynRange, @NotNull final PlainAxis yAxis )
    {
        final Layer layer = new Layer( "Layer", new Dimension2D( xsize, ysize ) );
        mainPane.add( layer );

        final CartesianGraph graph = new CartesianGraph( "Graph" );
        layer.setGraph( graph );
        graph.addYAxis( yAxis );

        final LinearTransform yt = new LinearTransform( new Range2D( ystart, yend ), ynRange );
        graph.setYTransform( yt );
        graph.setXTransform( xt );

        final LineAttribute lineAttribute = createRandomLineAttribute( megapoliPlot.getAxeType() );
        graph.setData( firstLine, lineAttribute );

        return graph;
    }

    /**
     * This method creates a List of SimpleLine with no data.
     * This list is used to display legend for these lines
     *
     * @param allLines
     * @param allLinesForSecondAxis
     * @param lines
     * @return
     */
    private List<SimpleLine> createEmptyLinesList( @NotNull final List<SimpleLine> allLines, @NotNull final List<SimpleLine> allLinesForSecondAxis, @NotNull final List<SimpleLine> lines )
    {
        final List<SimpleLine> emptyLinesList = allLines;
        emptyLinesList.removeAll( lines );
        emptyLinesList.addAll( allLinesForSecondAxis );
        emptyLinesList.removeAll( allLinesForSecondAxis );

        return emptyLinesList;
    }

    /**
     * This method display on the quicklook the legend for the lines with no data.
     *
     * @param lines
     * @param lkey
     * @param mainPane
     * @param xsize
     * @param ysize
     * @param xt
     * @param yt
     * @param axeType
     */
    private void displayLegendForEmptyLines( @NotNull final List<SimpleLine> lines, @NotNull final LineKey lkey, @NotNull final JPane mainPane, final double xsize, final double ysize, final LinearTransform xt, final LinearTransform yt, final String axeType )
    {
        for( final SimpleLine line : lines )
        {
            final Layer otherLayer = new Layer( "Other Layer", new Dimension2D( xsize, ysize ) );
            mainPane.add( otherLayer );
            final CartesianGraph otherGraph = new CartesianGraph( "Other Graph", xt, yt );
            otherLayer.setGraph( otherGraph );

            final LineAttribute otherLineAttribute = createRandomLineAttribute( axeType );
            otherGraph.setData( line, otherLineAttribute );
            lkey.addLineGraph( (LineCartesianRenderer) otherGraph.getRenderer(), line.getKeyTitle() );
        }
    }

    /**
     * This method display the copyright in the bottom right corner of the plot
     *
     * @param jPane
     * @param xSize
     * @param ySize
     * @param xPosition
     * @param yPosition
     */
    private void displayCopyright( @NotNull final JPane jPane, final double xSize, final double ySize, final double xPosition, final double yPosition )
    {
        final Layer copyrightLayer = new Layer( "CopyrightLayer", new Dimension2D( xSize, ySize ) );
        jPane.add( copyrightLayer );

        final Calendar calendar = Calendar.getInstance();
        final int year = calendar.get( Calendar.YEAR );
        final String copyright = "@" + year + _copyright;
        final SGLabel copyrightLabel = new SGLabel( "Copyright", copyright, new Point2D.Double( xPosition, yPosition ) );
        copyrightLabel.setAlign( SGLabel.TOP, SGLabel.CENTER );
        copyrightLabel.setFont( _copyrightFont );

        copyrightLayer.addChild( copyrightLabel );
    }

    /**
     * This method display the period's dates given by the user for a 2D plot
     *
     * @param jPane
     * @param beginDate
     * @param endDate
     */
    private void displayPeriod( @NotNull final JPane jPane, @Nullable final Date beginDate, @Nullable final Date endDate )
    {
        if( null != beginDate || null != endDate )
        {
            String periodStr = null != beginDate ? "[" + DateHelper.formatDate( beginDate, DateHelper.ENGLISH_DATE_PATTERN ) : "[X";
            periodStr += null != endDate ? " - " + DateHelper.formatDate( endDate, DateHelper.ENGLISH_DATE_PATTERN ) + "] (UTC)" : " - X] (UTC)";

            final Layer periodLayer = new Layer( "PeriodLayer", new Dimension2D( 1.25, 1.0 ) );
            jPane.add( periodLayer );

            final SGLabel periodLabel = new SGLabel( "Period", periodStr, new Point2D.Double( 0.1, 0.99 ) );
            periodLabel.setAlign( SGLabel.TOP, SGLabel.CENTER );
            periodLabel.setFont( _periodFont );

            periodLayer.addChild( periodLabel );
        }
    }

    /**
     * This method removes from a list the lines with no data
     *
     * @param allLines
     * @return
     */
    @NotNull
    private List<SimpleLine> extractRealLines( @NotNull final List<SimpleLine> allLines )
    {
        final List<SimpleLine> lines = new ArrayList<SimpleLine>();
        for( final SimpleLine line : allLines )
            if( 0 < line.getYArray().length )
                lines.add( line );

        return lines;
    }

    /**
     * This method creates a legend
     *
     * @param megapoliPlot
     * @param ysize
     * @param layer
     * @param firstLine
     * @param graph
     * @return
     */
    @NotNull
    private LineKey createLineKey( final MegapoliPlot megapoliPlot, final double ysize, final Layer layer, final SimpleLine firstLine, final CartesianGraph graph )
    {
        final LineKey lkey = new LineKey();
        if( !megapoliPlot.isLegendToHide() )
        {
            lkey.setId( "Legend" );
            lkey.setLocationP( new Point2D.Double( 0.1, ysize - 0.1 ) );
            lkey.setBorderStyle( LineKey.NO_BORDER );
            lkey.setVAlign( LineKey.TOP );
            lkey.setHAlign( LineKey.LEFT );
            layer.addChild( lkey );
            lkey.addLineGraph( (LineCartesianRenderer) graph.getRenderer(), firstLine.getKeyTitle() );
        }
        return lkey;
    }

    /**
     * This method creates a x-axis for time serie
     *
     * @param xnRange
     * @param originPoint
     * @return
     */
    @NotNull
    private TimeAxis createXTimeAxis( final SoTRange xnRange, final SoTPoint originPoint )
    {
        final TimeAxis xbot = new TimeAxis( "X-Axis", TimeAxis.AUTO );
        xbot.setRangeU( xnRange );
        xbot.setLocationU( originPoint );
        xbot.setLabelFont( _axisFont );
        xbot.setLabelHeightP( _heightAxisFont );
        // TimeZone indication
        xbot.setTitle( new SGLabel( "timeZone", " (UTC)", null ) );
//        xbot.setMajorLabelFormat( _majorLabelFormat );
        return xbot;
    }

    /**
     * This method creates a vertical axis, set its range in user units and its origin and creates the axis title
     *
     * @param line
     * @param ynRange
     * @param originPoint
     * @param labelPosition
     * @return
     */
    @NotNull
    private PlainAxis createYAxis( final SimpleLine line, final SoTRange ynRange, final SoTPoint originPoint, @Nullable final Integer labelPosition )
    {
        final PlainAxis yAxis = new PlainAxis( "Y-Axis" );
        yAxis.setRangeU( ynRange );
        yAxis.setLocationU( originPoint );
        yAxis.setLabelFont( _axisFont );
        if( null != labelPosition )
            yAxis.setLabelPosition( labelPosition );

        final String yLabel = line.getYMetaData().getName() + " (" + line.getYMetaData().getUnits() + ")";
        final SGLabel ytitle = new SGLabel( "Y-Axis Title", yLabel, new Point2D.Double( 0.0, 0.0 ) );
        ytitle.setFont( _axisFont );
        ytitle.setHeightP( _heightAxisFont );
        yAxis.setTitle( ytitle );
        return yAxis;
    }

    /**
     * This method adds layers corresponding to the other lines to display
     *
     * @param axeType
     * @param mainPane
     * @param xsize
     * @param ysize
     * @param line
     * @param xt
     * @param yt
     * @param lkey
     */
    private void addLineInNewGraph( @NotNull final String axeType, @NotNull final JPane mainPane, final double xsize, final double ysize, @NotNull final SimpleLine line, @NotNull final LinearTransform xt, @NotNull final LinearTransform yt, @NotNull final LineKey lkey )
    {
        final Layer otherLayer = new Layer( "Other Layer", new Dimension2D( xsize, ysize ) );
        mainPane.add( otherLayer );
        final CartesianGraph otherGraph = new CartesianGraph( "Other Graph", xt, yt );
        otherLayer.setGraph( otherGraph );

        final LineAttribute otherLineAttribute = createRandomLineAttribute( axeType );
        otherGraph.setData( line, otherLineAttribute );
        lkey.addLineGraph( (LineCartesianRenderer) otherGraph.getRenderer(), line.getKeyTitle() );
    }

    /**
     * This method returns a new LineAttribute with a random color
     *
     * @param axeType
     * @return
     */
    @NotNull
    private LineAttribute createRandomLineAttribute( @NotNull final String axeType )
    {
        final LineAttribute attr;
        final Color color;
        if( _indexColorArray >= _colorsArray.size() )
            color = new Color( new Random().nextInt( 256 ), new Random().nextInt( 256 ), new Random().nextInt( 256 ) );
        else
            color = _colorsArray.get( _indexColorArray );
        _indexColorArray++;

        if( axeType.equals( AxeTypeForFixedPlateform.TIME_POINTS.toString() ) )
            attr = new LineAttribute( LineAttribute.MARK, 1, color );
        else
            attr = new LineAttribute( LineAttribute.SOLID, 1, color );

        attr.setMarkHeightP( _markHeight );

        return attr;
    }

    /**
     * This method extracts from a list of <plateformId, parameterId> two lists of <plateformId, parameterId> corresponding for the 2 axes needed for the graph
     * The second list can be empty, in this case, only one axis is needed
     *
     * @param pfIdPIdList
     * @return
     */
    @NotNull
    private Pair<List, List> extractListPairForAxes( @NotNull final List<Pair<Integer, Integer>> pfIdPIdList )
    {
        final List<Pair<Integer, Integer>> pIdPIdListForFirstAxis = new ArrayList<Pair<Integer, Integer>>();
        final List<Pair<Integer, Integer>> pIdPIdListForSecondAxis = new ArrayList<Pair<Integer, Integer>>();

        for( final Pair pIdPId : pfIdPIdList )
        {
            final List<Integer> secondValuesForFirstAxis = EtherHelper.getSecondValues( pIdPIdListForFirstAxis );
            final Integer parameterId = (Integer) pIdPId.getSecondValue();

            if( pIdPIdListForFirstAxis.isEmpty() || secondValuesForFirstAxis.contains( parameterId ) )
                pIdPIdListForFirstAxis.add( pIdPId );
            else
                pIdPIdListForSecondAxis.add( pIdPId );
        }

        return new Pair( pIdPIdListForFirstAxis, pIdPIdListForSecondAxis );
    }

    /**
     * This method extracts data from BD for the plateform and parameter given as parameters
     * and creates the corresponded simpleLine
     * Some lines can be empty if there's no data in this period by example
     *
     * @param pIdPIdList
     * @param beginDate
     * @param endDate
     * @param locale
     * @return
     * @throws ServiceException
     */
    @NotNull
    private List<SimpleLine> createLines( @NotNull final List<Pair> pIdPIdList, @Nullable final Date beginDate, @Nullable final Date endDate, final Integer delta, final Locale locale )
            throws ServiceException
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );
        final String messageData = bundle.getString( "plot.noData" );

        final List<SimpleLine> lines = new ArrayList<SimpleLine>( pIdPIdList.size() );
        boolean isFirstLine = true;

        for( final Pair<Integer, Integer> pIdPId : pIdPIdList )
        {
            final Integer plateformId = pIdPId.getFirstValue();
            final Integer parameterId = pIdPId.getSecondValue();

            final Plateforme plateform = _etherService.getPlateformById( plateformId );
//            final Plateforme plateform = BouchonHelper.getPlateformById( plateformId );
            if( null == plateform )
                throw new ServiceException( ServiceException.ServiceCode.PLATEFORM_IS_NULL, new Throwable( ServiceException.ServiceCode.PLATEFORM_IS_NULL.toString() ) );

            final Parametre parametre = _etherService.getParameterById( parameterId );
//            final Parametre parametre = BouchonHelper.getParameterById( parameterId );
            if( null == parametre )
                throw new ServiceException( ServiceException.ServiceCode.PARAMETER_IS_NULL, new Throwable( ServiceException.ServiceCode.PARAMETER_IS_NULL.toString() ) );

            final Data valuesLists = _etherService.getListsByPlateformByParameterByPeriodForTimeSerie( plateformId, parameterId, beginDate, endDate );
//            final Data valuesLists = BouchonHelper.createValuesForTimeSerie( 50, delta );

            final double[] dataArray = (double[]) valuesLists.getFirstArray();
            final Date[] dateValues = (Date[]) valuesLists.getSecondArray();

            final SimpleLine line = new SimpleLine( new GeoDateArray( dateValues ), dataArray, "legend" );
            SGTMetaData meta = new SGTMetaData( "", "", false, false );
            line.setXMetaData( meta );
            final SGLabel sgLabel;
            if( 0 < dataArray.length && 0 < dateValues.length )
                sgLabel = new SGLabel( "Line", plateform.getPlateformeNom() + "-" + parametre.getParametreNom(), new Point2D.Double( 0.0, 0.0 ) );
            else
                sgLabel = new SGLabel( "Line", plateform.getPlateformeNom() + "-" + parametre.getParametreNom() + " " + messageData, new Point2D.Double( 0.0, 0.0 ) );
            line.setKeyTitle( sgLabel );

            if( isFirstLine )
                meta = new SGTMetaData( parametre.getParametreNom(), parametre.getUnite().getUniteCode(), false, false );
            else
                meta = new SGTMetaData( "", "", false, false );
            line.setYMetaData( meta );

            lines.add( line );
            isFirstLine = false;
        }
        return lines;
    }

    @NotNull
    private ColorMap createColorMap( final Range2D datar )
    {
        final int[] red =
                {0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0,
                        0, 7, 23, 39, 55, 71, 87, 103,
                        119, 135, 151, 167, 183, 199, 215, 231,
                        247, 255, 255, 255, 255, 255, 255, 255,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 246, 228, 211, 193, 175, 158, 140};
        final int[] green =
                {0, 0, 0, 0, 0, 0, 0, 0,
                        0, 11, 27, 43, 59, 75, 91, 107,
                        123, 139, 155, 171, 187, 203, 219, 235,
                        251, 255, 255, 255, 255, 255, 255, 255,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 247, 231, 215, 199, 183, 167, 151,
                        135, 119, 103, 87, 71, 55, 39, 23,
                        7, 0, 0, 0, 0, 0, 0, 0};
        final int[] blue =
                {0, 143, 159, 175, 191, 207, 223, 239,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 255, 255, 255, 255, 255, 255, 255,
                        255, 247, 231, 215, 199, 183, 167, 151,
                        135, 119, 103, 87, 71, 55, 39, 23,
                        7, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0,
                        0, 0, 0, 0, 0, 0, 0, 0};

        final IndexedColorMap cmap = new IndexedColorMap( red, green, blue );
        cmap.setTransform( new LinearTransform( 0.0, (double) red.length, datar.start, datar.end ) );
        return cmap;
    }

    public EtherService getEtherService()
    {
        return _etherService;
    }

    @Required
    public void setEtherService( final EtherService etherService )
    {
        _etherService = etherService;
    }

    private static Integer _indexColorArray = 0;
    private static final List<Color> _colorsArray = new ArrayList<Color>( Arrays.asList( new Color( 0, 204, 204 ), new Color( 255, 0, 0 ), new Color( 102, 102, 204 ), new Color( 51, 255, 51 ),
            new Color( 70, 130, 180 ), new Color( 178, 34, 34 ), new Color( 148, 0, 211 ), new Color( 193, 205, 201 ),
            new Color( 0, 139, 69 ), new Color( 255, 215, 0 ), new Color( 255, 165, 79 ), new Color( 54, 54, 54 ) ) );

    // Dimensions of the jPanes
    protected static final int MAX_WIDTH = 800;
    protected static final int MAX_HEIGHT = 700;
    protected static final int MAX_HEIGHT_TOP = 100;
    protected static final int MARGIN_LEFT_RIGHT = 50;
    protected static final int LEGEND_2D_HEIGHT = 150;

    protected static final int TITLE_FONT_SIZE = 20;
    protected static final int FONT_SIZE = 12;
    protected static final int ERROR_FONT_SIZE = 10;

    protected static final Font _axisFont = new Font( "Helvetica", Font.PLAIN, 15 );
    protected static final Font _copyrightFont = new Font( "Helvetica", Font.PLAIN, 8 );
    protected static final Font _periodFont = new Font( "Helvetica", Font.PLAIN, 10 );
    protected static final String _copyright = " CNES-CNRS";
    protected static final double _heightAxisFont = 0.14;
    protected static final double _markHeight = 0.1;
    protected static final String _majorLabelFormat = "yyyy-MM-dd HH:mm";
    // Number of intervals on a Y-Axis
    protected Integer _intervalsNumber = 10;
    protected Integer _numberOfColorsFor2D = 64;

    private EtherService _etherService;
}
